import pyamdgpuinfo
import nvsmi
from shutil import which


class Addon():

    def __init__(self, lnxlink):
        self.name = 'GPU'
        self.gpu_ids = {
            "amd": pyamdgpuinfo.detect_gpus()
        }
        if which("nvidia-smi") is not None:
            self.gpu_ids["nvidia"] = len(list(nvsmi.get_gpus()))
        else:
            self.gpu_ids["nvidia"] = 0

    def getControlInfo(self):
        gpus = {}
        for gpu_id in range(self.gpu_ids['amd']):
            amd_gpu = pyamdgpuinfo.get_gpu(gpu_id)
            gpus[f"amd_{gpu_id}"] = {
                "name": amd_gpu.name,
                "VRAM usage": amd_gpu.query_vram_usage(),
                "GTT usage": amd_gpu.query_gtt_usage(),
                "Temperature": amd_gpu.query_temperature(),
                "load": min(100, amd_gpu.query_load() * 100),
                "Power": amd_gpu.query_power(),
                "Voltage": amd_gpu.query_graphics_voltage(),
            }
        for gpu_id in range(self.gpu_ids['nvidia']):
            nvidia_gpu = list(nvsmi.get_gpus())[gpu_id]
            gpus[f"nvidia_{gpu_id}"] = {
                "name": nvidia_gpu.name,
                "Memory usage": round(nvidia_gpu.mem_util, 1),
                "load": min(100, round(nvidia_gpu.gpu_util, 1)),
                "Temperature": nvidia_gpu.temperature,
            }
        return gpus

    def exposedControls(self):
        discovery_info = {}
        for gpu_id in range(self.gpu_ids['amd']):
            discovery_info[f"GPU AMD {gpu_id}"] = {
                "type": "sensor",
                "icon": "mdi:expansion-card-variant",
                "unit": "%",
                "state_class": "measurement",
                "value_template": f"{{{{ value_json.amd_{gpu_id}.load }}}}",
                "enabled": True,
            }
        for gpu_id in range(self.gpu_ids['nvidia']):
            discovery_info[f"GPU NVIDIA {gpu_id}"] = {
                "type": "sensor",
                "icon": "mdi:expansion-card-variant",
                "unit": "%",
                "state_class": "measurement",
                "value_template": f"{{{{ value_json.nvidia_{gpu_id}.load }}}}",
                "enabled": True,
            }
        return discovery_info



import psutil


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Disk Usage'

    def exposedControls(self):
        discovery_info = {}
        for disk in psutil.disk_partitions():
            if disk.fstype == 'squashfs':
                continue
            if 'docker/overlay' in disk.mountpoint:
                continue
            device = disk.device.replace('/', '_').strip('_')
            discovery_info[f"Disk {device}"] = {
                "type": "sensor",
                "icon": "mdi:harddisk",
                "unit": "%",
                "state_class": "measurement",
                "value_template": f"{{{{ value_json.{device}.percent }}}}",
                "enabled": True,
            }
        return discovery_info

    def getControlInfo(self):
        disks = {}
        for disk in psutil.disk_partitions():
            if disk.fstype == 'squashfs':
                continue
            if 'docker/overlay' in disk.mountpoint:
                continue
            device = disk.device.replace('/', '_').strip('_')
            disks[device] = {}
            disk_stats = psutil.disk_usage(disk.mountpoint)
            disks[device]["total"] = self._bytetomb(disk_stats.total)
            disks[device]["used"] = self._bytetomb(disk_stats.used)
            disks[device]["free"] = self._bytetomb(disk_stats.free)
            disks[device]["percent"] = disk_stats.percent
        return disks

    def _bytetomb(self, byte):
        return round(byte / 1024 / 1024, 1)



import subprocess
import re


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Keep Alive'
        self.keepalive = 'OFF'

    def exposedControls(self):
        return {
            "Keep Alive": {
                "type": "switch",
                "icon": "mdi:mouse-variant",
            }
        }

    def getControlInfo(self):
        enabled_list = []

        # Check if Gnome Idle Time is active
        stdout_dim = subprocess.run(
            'gsettings get org.gnome.desktop.session idle-delay',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        stdout_suspend = subprocess.run(
            'gsettings get org.gnome.settings-daemon.plugins.power sleep-inactive-ac-type',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        if stdout_dim != '':
            disabled_dim = '0' == stdout_dim.split()[1]
            if disabled_dim and stdout_suspend != '':
                enabled_list.append('nothing' in stdout_suspend)

        # Check if DPMS is active
        stdout_xset = subprocess.run(
            'xset q',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        xset_pattern = re.compile(r"Standby: (\d+)\s+Suspend: (\d+)\s+Off: (\d+)")
        xset_match = re.findall(xset_pattern, stdout_xset)
        for nums in xset_match:
            enabled_list.append(all(num != '0' for num in nums))
            if enabled_list[-1]:
                enabled_list.append('DPMS is Enabled' in stdout_xset)

        return any(enabled_list)

    def startControl(self, topic, data):
        if data.lower() == 'off':
            subprocess.run(
                'gsettings set org.gnome.desktop.session idle-delay 600',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            subprocess.run(
                'gsettings set org.gnome.settings-daemon.plugins.power sleep-inactive-ac-type "suspend"',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            subprocess.run(
                'xset +dpms',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        elif data.lower() == 'on':
            subprocess.run(
                'gsettings set org.gnome.desktop.session idle-delay 0',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            subprocess.run(
                'gsettings set org.gnome.settings-daemon.plugins.power sleep-inactive-ac-type "nothing"',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            subprocess.run(
                'xset -dpms',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)



from ewmh import EWMH


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Fullscreen'

    def exposedControls(self):
        return {
            "Fullscreen": {
                "type": "binary_sensor",
                "icon": "mdi:alert-octagon-outline",
                "value_template": "{{ value_json.is_fullscreen }}",
            },
        }

    def getControlInfo(self):
        data = {
            "is_fullscreen": "OFF",
            "window": "",
        }
        ewmh = EWMH()
        wins = ewmh.getClientList()
        for w in wins:
            state = ewmh.getWmState(w, True)
            name = ewmh.getWmName(w)
            if "_NET_WM_STATE_FULLSCREEN" in state:
                data["is_fullscreen"] = "ON"
                data["window"] = name.decode()

        return data
